@@ -12,4 +12,7 @@ pub use error::*;
 mod parser;
 pub use parser::*;
 
+mod tokenizer;
+pub use tokenizer::*;
+
 pub const VERSION: &str = env!("CARGO_PKG_VERSION");
